@@ -7,7 +7,7 @@ import collections
 import ctypes
 import ctypes.util
 import os
-from subprocess import CalledProcessError, call
+import subprocess
 from tempfile import mkstemp
 
 from .configgenerator import ConfigGenerator
@@ -153,7 +153,8 @@ class NSSGenerator(ConfigGenerator):
         'SECP384R1': 'SECP384R1',
         'SECP521R1': 'SECP521R1',
         'MLKEM768-X25519': 'mlkem768x25519',
-        'P256-MLKEM768': 'mlkem768secp256r1',
+        'P256-MLKEM768': 'secp256r1mlkem768',
+        'P384-MLKEM1024': 'secp384r1mlkem1024',
     }
 
     cipher_map = {
@@ -208,6 +209,9 @@ class NSSGenerator(ConfigGenerator):
         'ECDSA-': 'ECDSA',
         'DSA-': 'DSA',
         'EDDSA-ED25519': 'ED25519',
+        'MLDSA44': 'ML-DSA-44',
+        'MLDSA65': 'ML-DSA-65',
+        'MLDSA87': 'ML-DSA-87',
     }
     sign_suffix_ordmap = {
         '-MD5': 'MD5',
@@ -412,29 +416,42 @@ class NSSGenerator(ConfigGenerator):
         except AttributeError:
             cls.eprint('Cannot determine nss version with ctypes, '
                        'assuming >=3.80')
-        options = ('-f value -f identifier'
-                   if nss_is_lax_by_default and not nss_lax else '')
+        options = (['-f', 'value', '-f', 'identifier']
+                   if nss_is_lax_by_default and not nss_lax else [])
 
         fd, path = mkstemp()
 
-        ret = 255
         try:
             with os.fdopen(fd, 'w') as f:
                 f.write(config)
-            try:
-                ret = call(f'/usr/bin/nss-policy-check {options} {path}'
-                           '>/dev/null',
-                           shell=True)
-            except CalledProcessError:
-                cls.eprint("/usr/bin/nss-policy-check: Execution failed")
+            p = subprocess.run(['nss-policy-check',  # noqa: S607
+                                *options, path],
+                               check=False,
+                               encoding='utf-8',
+                               stdout=subprocess.PIPE,
+                               stderr=subprocess.STDOUT)
         finally:
             os.unlink(path)
 
-        if ret == 2:
+        if p.returncode == 1:  # NSS-POLICY-WARN
+            # FUTURE policy currently defines no classic KEX, this is intended
+            lines = p.stdout.split('\n')
+            for line in ('NSS-POLICY-WARN: NUMBER-OF-ECC: 0',
+                         'NSS-POLICY-WARN: NUMBER-OF-SMIME-KX: 0',
+                         'NSS-POLICY-WARN'):
+                if line in lines:
+                    lines.remove(line)
+            rest = '\n'.join(lines)
+            if 'NSS-POLICY-FAIL' not in rest and 'NSS-POLICY-WARN' not in rest:
+                # OK, that's an acceptable warning
+                return True
+            # otherwise that's some other warning
+            cls.eprint("/usr/bin/nss-policy-check: Execution failed")
             cls.eprint("There is a warning in NSS generated policy")
             cls.eprint(f'Policy:\n{config}')
             return False
-        if ret:
+        if p.returncode:  # NSS-POLICY-FAIL and catchall for unexpected non-0
+            cls.eprint("/usr/bin/nss-policy-check: Execution failed")
             cls.eprint("There is an error in NSS generated policy")
             cls.eprint(f'Policy:\n{config}')
             return False
